@@ -11,8 +11,8 @@ use raw::KeyType;
 use crate::from_byte_string;
 use crate::native_types::RedisType;
 use crate::raw;
-use crate::RedisError;
 use crate::redismodule::REDIS_OK;
+use crate::RedisError;
 use crate::RedisResult;
 use crate::RedisString;
 
@@ -37,20 +37,19 @@ pub struct RedisKey {
 }
 
 impl RedisKey {
-    pub fn open(ctx: *mut raw::RedisModuleCtx, key: &str) -> RedisKey {
-        let key_str = RedisString::create(ctx, key);
-        let key_inner = raw::open_key(ctx, key_str.inner, to_raw_mode(KeyMode::Read));
-        RedisKey {
-            ctx: ctx,
-            key_inner: key_inner,
-        }
+    pub fn open(ctx: *mut raw::RedisModuleCtx, key: &RedisString) -> RedisKey {
+        let key_inner = raw::open_key(ctx, key.inner, to_raw_mode(KeyMode::Read));
+        RedisKey { ctx, key_inner }
     }
 
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_ModuleTypeGetValue` is missing in redismodule.h
     pub fn get_value<T>(&self, redis_type: &RedisType) -> Result<Option<&T>, RedisError> {
         verify_type(self.key_inner, redis_type)?;
 
         let value =
-            unsafe { raw::RedisModule_ModuleTypeGetValue.unwrap()(self.key_inner) as *mut T };
+            unsafe { raw::RedisModule_ModuleTypeGetValue.unwrap()(self.key_inner).cast::<T>() };
 
         if value.is_null() {
             return Ok(None);
@@ -61,6 +60,9 @@ impl RedisKey {
         Ok(Some(value))
     }
 
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_KeyType` is missing in redismodule.h
     pub fn key_type(&self) -> raw::KeyType {
         unsafe { raw::RedisModule_KeyType.unwrap()(self.key_inner) }.into()
     }
@@ -129,13 +131,9 @@ pub struct RedisKeyWritable {
 }
 
 impl RedisKeyWritable {
-    pub fn open(ctx: *mut raw::RedisModuleCtx, key: &str) -> RedisKeyWritable {
-        let key_str = RedisString::create(ctx, key);
-        let key_inner = raw::open_key(ctx, key_str.inner, to_raw_mode(KeyMode::ReadWrite));
-        RedisKeyWritable {
-            ctx: ctx,
-            key_inner: key_inner,
-        }
+    pub fn open(ctx: *mut raw::RedisModuleCtx, key: &RedisString) -> RedisKeyWritable {
+        let key_inner = raw::open_key(ctx, key.inner, to_raw_mode(KeyMode::ReadWrite));
+        RedisKeyWritable { ctx, key_inner }
     }
 
     /// Detects whether the value stored in a Redis key is empty.
@@ -255,11 +253,17 @@ impl RedisKeyWritable {
         }
     }
 
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_DeleteKey` is missing in redismodule.h
     pub fn delete(&self) -> RedisResult {
         unsafe { raw::RedisModule_DeleteKey.unwrap()(self.key_inner) };
         REDIS_OK
     }
 
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_KeyType` is missing in redismodule.h
     pub fn key_type(&self) -> raw::KeyType {
         unsafe { raw::RedisModule_KeyType.unwrap()(self.key_inner) }.into()
     }
@@ -273,16 +277,19 @@ impl RedisKeyWritable {
         key: *mut raw::RedisModuleString,
     ) -> RedisKeyWritable {
         let key_inner = raw::open_key(ctx, key, to_raw_mode(KeyMode::ReadWrite));
-        RedisKeyWritable {
-            ctx: ctx,
-            key_inner: key_inner,
-        }
+        RedisKeyWritable { ctx, key_inner }
     }
 
-    pub fn get_value<'a, 'b, T>(&'a self, redis_type: &RedisType) -> Result<Option<&'b mut T>, RedisError> {
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_ModuleTypeGetValue` is missing in redismodule.h    
+    pub fn get_value<'a, 'b, T>(
+        &'a self,
+        redis_type: &RedisType,
+    ) -> Result<Option<&'b mut T>, RedisError> {
         verify_type(self.key_inner, redis_type)?;
         let value =
-            unsafe { raw::RedisModule_ModuleTypeGetValue.unwrap()(self.key_inner) as *mut T };
+            unsafe { raw::RedisModule_ModuleTypeGetValue.unwrap()(self.key_inner).cast::<T>() };
 
         if value.is_null() {
             return Ok(None);
@@ -292,9 +299,12 @@ impl RedisKeyWritable {
         Ok(Some(value))
     }
 
+    /// # Panics
+    ///
+    /// Will panic if `RedisModule_ModuleTypeSetValue` is missing in redismodule.h
     pub fn set_value<T>(&self, redis_type: &RedisType, value: T) -> Result<(), RedisError> {
         verify_type(self.key_inner, redis_type)?;
-        let value = Box::into_raw(Box::new(value)) as *mut c_void;
+        let value = Box::into_raw(Box::new(value)).cast::<c_void>();
         let status: raw::Status = unsafe {
             raw::RedisModule_ModuleTypeSetValue.unwrap()(
                 self.key_inner,
@@ -366,7 +376,7 @@ where
 
     /// Provides an iterator over the multi-get results in the form of (field-name, field-value)
     /// pairs. The type of field-name elements is the same as that passed to the original multi-
-    /// get call, while the field-value elements may be of any type for which a RedisString `Into`
+    /// get call, while the field-value elements may be of any type for which a `RedisString` `Into`
     /// conversion is implemented.  
     ///
     /// # Examples
@@ -470,6 +480,10 @@ fn to_raw_mode(mode: KeyMode) -> raw::KeyMode {
     }
 }
 
+/// # Panics
+///
+/// Will panic if `RedisModule_KeyType` or `RedisModule_ModuleTypeGetType` are missing in redismodule.h
+#[allow(clippy::not_unsafe_ptr_arg_deref)]
 pub fn verify_type(key_inner: *mut raw::RedisModuleKey, redis_type: &RedisType) -> RedisResult {
     let key_type: KeyType = unsafe { raw::RedisModule_KeyType.unwrap()(key_inner) }.into();
 
