@@ -109,6 +109,13 @@ macro_rules! redis_module {
             ]),* $(,)*
         ])?
     ) => {
+        extern "C" fn __info_func(
+            ctx: *mut $crate::raw::RedisModuleInfoCtx,
+            for_crash_report: i32,
+        ) {
+            $crate::base_info_func(ctx, for_crash_report == 1);
+        }
+
         #[no_mangle]
         #[allow(non_snake_case)]
         pub extern "C" fn RedisModule_OnLoad(
@@ -168,6 +175,8 @@ macro_rules! redis_module {
                 )*
             )?
 
+            raw::register_info_function(ctx, Some(__info_func));
+
             raw::Status::Ok as c_int
         }
 
